SystemOrganization addCategory: #'Seaside-Session'!
SystemOrganization addCategory: #'Seaside-Session-Filter'!

WAObject subclass: #WACurrentSession
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Session'!

!WACurrentSession commentStamp: 'jf 9/11/2008 14:58' prior: 0!
Available for backward compatibility. You should now use "self session" in any subclass of WAObject or "WACurrentRequestContext value session" anywhere else.!

!WACurrentSession class methodsFor: 'evaluating' stamp: 'lr 12/30/2008 12:48'!
value
	| context |
	self 
		seasideDeprecatedApi: 'WACurrentSession class>>#value'
		details: 'Use "self session" on a subclass of WAObject or "WACurrentRequestContext value session".'.
	^ (context := WACurrentRequestContext value) isNil ifFalse: [ context session ]! !

WAObject subclass: #WAMain
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Session'!

!WAMain commentStamp: 'rew 10/19/2007 10:49' prior: 0!
WAMain subclasses are used to initialize applications when a new session is started.

Subclasses must implement the following messages:
	start:
		Does the initializing 
!

!WAMain methodsFor: 'processing' stamp: 'jf 10/19/2008 11:53'!
start
	"The main entry point into a session."

	self subclassResponsibility! !

!WAObject methodsFor: '*seaside-session' stamp: 'pmm 9/1/2008 13:27'!
application
	"Answer the current seaside application, instance of WAApplication or a subclass."
	
	^self requestContext application! !

!WAObject methodsFor: '*seaside-session' stamp: 'pmm 9/1/2008 13:26'!
session
	"Answer the current seaside session, instance of WASesssion or a subclass."
	
	^self requestContext session! !

WAObject subclass: #WASessionContinuation
	instanceVariableNames: 'states'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Session'!

!WASessionContinuation commentStamp: 'lr 6/2/2008 09:19' prior: 0!
I represent a continuation as part of the flow of pages within a session. I am not a real continuation (as compared to those used in #call: and #answer:), I only represent a specific point in the session. I reference the root component and a memory snapshot of backtracked objects.!

!WASessionContinuation methodsFor: 'evaluating' stamp: 'jf 1/21/2009 14:45'!
basicValue
	self withUnregisteredHandlerDo: [ self handleRequest ]! !

!WASessionContinuation methodsFor: 'evaluating' stamp: 'jf 1/21/2009 14:44'!
captureAndInvoke
	self captureState.
	self basicValue! !

!WASessionContinuation methodsFor: 'processing' stamp: 'jf 1/21/2009 14:11'!
captureState
	"self states reset"
	"Setting states to nil matches the behaviour of Seaside 2.8. Resetting the Snapshot
	is useful for JS callbacks in that refreshing the page will show the modified state
	but can also result in many ActionPhaseContinuations all sharing the same state
	but with different callbacks, some of which may reference state that is no longer present."
	states := nil.
	self updateStates: self states! !

!WASessionContinuation methodsFor: 'processing' stamp: 'jf 10/19/2008 12:35'!
handleRequest
	self subclassResponsibility! !

!WASessionContinuation methodsFor: 'private' stamp: 'lr 1/19/2009 19:09'!
redirectToContinuation: aSessionContinuation
	self respond: [ :response | response redirectTo: aSessionContinuation registerForUrl ]! !

!WASessionContinuation methodsFor: 'private' stamp: 'jf 1/11/2009 17:39'!
registerForUrl
	| url |
	url := self session actionUrlForContinuation: self.
	self updateUrl: url.
	^ url! !

!WASessionContinuation methodsFor: 'accessing' stamp: 'lr 4/9/2008 10:52'!
request
	"Answer the current request object."

	^ self requestContext request! !

!WASessionContinuation methodsFor: 'processing' stamp: 'lr 1/19/2009 18:23'!
respond: aBlock 
	| context |
	context := self requestContext.
	aBlock value: context response.
	context respond! !

!WASessionContinuation methodsFor: 'initialization' stamp: 'jf 10/16/2008 15:32'!
setStates: aSnapshot
	states := aSnapshot! !

!WASessionContinuation methodsFor: 'accessing' stamp: 'lr 1/19/2009 23:00'!
states
	^ states ifNil: [ states := WASnapshot new ]! !

!WASessionContinuation methodsFor: 'updating' stamp: 'jf 9/28/2008 23:07'!
updateRoot: aHtmlRoot
	self session updateRoot: aHtmlRoot! !

!WASessionContinuation methodsFor: 'updating' stamp: 'jf 9/28/2008 23:07'!
updateStates: aSnapshot
	self session updateStates: aSnapshot! !

!WASessionContinuation methodsFor: 'updating' stamp: 'jf 9/28/2008 23:06'!
updateUrl: anUrl
	self session updateUrl: anUrl! !

!WASessionContinuation methodsFor: 'deprecated' stamp: 'lr 1/25/2009 12:00'!
url
	self
		seasideDeprecatedApi: 'WASessionContinuation>>#url'
		details: 'Use WASessionContinuation>>registerForUrl.'.
	^ self registerForUrl! !

!WASessionContinuation methodsFor: 'evaluating' stamp: 'jf 1/21/2009 14:45'!
value
	"Resume processing of a request. To ensure valid application state restore all registered states."

	self states restore.
	self basicValue! !

!WASessionContinuation methodsFor: 'private' stamp: 'lr 11/14/2008 23:36'!
withUnregisteredHandlerDo: aBlock 
	^ aBlock 
		on: WAUnregisteredHandlerError
		do: [ :err | self requestContext redirectTo: self application baseUrl ]! !

WAObject subclass: #WASnapshot
	instanceVariableNames: 'objects'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Session'!

!WASnapshot commentStamp: 'lr 6/2/2008 09:24' prior: 0!
I represent a memory snapshot of objects. Objects can be registered by sending the message #register:. All object snapshots are updated by sending #snapshot and restored by sending #restore. To have specific snpshot and restore actions for particular classes override #snapshotCopy and #restoreFromSnapshot: in your classes.!

!WASnapshot methodsFor: 'initialization' stamp: 'lr 1/19/2009 23:10'!
initialize
	super initialize.
	self reset! !

!WASnapshot methodsFor: 'registry' stamp: 'lr 1/19/2009 23:04'!
register: anObject
	"Register anObject for backtracking."

	anObject isNil
		ifFalse: [ objects at: anObject put: anObject snapshotCopy ]! !

!WASnapshot methodsFor: 'actions' stamp: 'lr 1/19/2009 23:10'!
reset
	"Clears the snapshot dictionary."

	objects := IdentityDictionary new! !

!WASnapshot methodsFor: 'actions' stamp: 'lr 5/31/2008 16:28'!
restore
	"Restore all the backtracked states."

	objects keysAndValuesDo: [ :key :value | key restoreFromSnapshot: value ]! !

!WASnapshot methodsFor: 'actions' stamp: 'lr 5/31/2008 16:29'!
snapshot
	"Snapshot all the states that have been registered for backtracking overriding existing snapshots."

	objects associationsDo: [ :assoc | assoc value: assoc key snapshotCopy ]! !

!WARequestContext methodsFor: '*seaside-session' stamp: 'jf 7/16/2008 10:49'!
application
	"Answer the current application or nil."

	^ handlers 
		detect: [ :each | each isApplication ]
		ifNone: [ nil ]! !

!WARequestContext methodsFor: '*seaside-session' stamp: 'jf 1/21/2009 10:51'!
redirect
	^ self respondAndContinue: [ :url | self redirectTo: url ]! !

!WARequestContext methodsFor: '*seaside-session' stamp: 'jf 1/21/2009 10:40'!
redirectTo: aUrlOrString
	self responseFactory redirectTo: aUrlOrString; respond! !

!WARequestContext methodsFor: '*seaside-session' stamp: 'lr 1/25/2009 12:13'!
redirectWithCookie: aCookie
	self 
		seasideDeprecatedApi: 'WARequestContext>>#redirectWithCookie:'
		details: 'Do self requestContent response #addCooke:.'.
	self respondAndContinue: [ :url |
		self response addCookie: aCookie.
		self redirectTo: url ]! !

!WARequestContext methodsFor: '*seaside-session' stamp: 'lr 1/12/2009 09:07'!
redirectWithMessage: aString delay: aNumber 
	self respondAndContinue: [ :url | 
		self responseFactory
			refreshAfter: aNumber
			location: url
			message: aString ]! !

!WARequestContext methodsFor: '*seaside-session' stamp: 'jf 1/21/2009 10:48'!
respondAndContinue: aBlock 
	^ WAContinuation currentDo: [ :cc |
		aBlock value: (self session actionUrlForContinuation: cc).
		self respond ]! !

!WARequestContext methodsFor: '*seaside-session' stamp: 'jf 7/25/2008 20:34'!
session
	"Answer the current session or nil."

	^ handlers 
		detect: [ :each | each isSession ]
		ifNone: [ nil ]! !

WARequestFilter subclass: #WAEscapeFilter
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Session-Filter'!

!WAEscapeFilter methodsFor: 'processing' stamp: 'lr 1/12/2009 13:58'!
handleFiltered: aRequestContext
	"Setup a return response escape handler. This is simply a marker in the execution stack."

	[ super handleFiltered: aRequestContext ]
		on: WAResponseNotification
		do: [ :err | ^ self ].
	self error: self printString , ' did not return a response.'! !

WARequestFilter subclass: #WAExceptionFilter
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Session-Filter'!

!WAExceptionFilter methodsFor: 'private' stamp: 'lr 11/3/2008 21:37'!
exceptionHandlerForContext: aRequestContext
	^ aRequestContext application preferenceAt: #exceptionHandler! !

!WAExceptionFilter methodsFor: 'processing' stamp: 'lr 1/12/2009 08:59'!
handleFiltered: aRequestContext
	(self exceptionHandlerForContext: aRequestContext)
		handleExceptionsDuring: [ super handleFiltered: aRequestContext ]
		context: aRequestContext! !

WARequestFilter subclass: #WAMonitorFilter
	instanceVariableNames: 'monitor'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Session-Filter'!

!WAMonitorFilter methodsFor: 'processing' stamp: 'lr 1/19/2009 18:24'!
handleFiltered: aRequestContext
	"The monitor should never get any errors, because the exception handler in the main block should catch any errors that come up. But if it does not, then we fall back to the lowest common denominator here."
	
	(self shouldTerminate: aRequestContext)
		ifTrue: [ self terminate: aRequestContext ]
		ifFalse: [
			monitor
				critical: [ super handleFiltered: aRequestContext ]
				ifError: [ :error | 
					WAExceptionHandler
						internalError: error 
						context: aRequestContext ].
			aRequestContext respond ]! !

!WAMonitorFilter methodsFor: 'initialization' stamp: 'lr 11/3/2008 21:32'!
initialize
	super initialize.
	monitor := WAProcessMonitor new! !

!WAMonitorFilter methodsFor: 'private' stamp: 'lr 11/12/2008 08:20'!
shouldTerminate: aRequestContext
	^ (aRequestContext request fields includesKey: 'terminate')
		and: [ aRequestContext application preferenceAt: #sessionAllowTermination ]! !

!WAMonitorFilter methodsFor: 'private' stamp: 'lr 1/19/2009 18:24'!
terminate: aRequestContext
	monitor terminate.
	aRequestContext response
		contentType: WAMimeType textPlain;
		nextPutAll: 'Process terminated'.
	aRequestContext respond! !

WARequestFilter subclass: #WAMutualExclusionFilter
	instanceVariableNames: 'semaphore owner'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Session-Filter'!

!WAMutualExclusionFilter methodsFor: 'private' stamp: 'jf 1/14/2009 13:32'!
critical: aBlock
	| process |
	process := Processor activeProcess.
	(owner == process)
		ifTrue: [ aBlock value ]
		ifFalse: [
			semaphore critical: [
				owner := process.
				aBlock ensure: [ owner := nil ] ] ]! !

!WAMutualExclusionFilter methodsFor: 'processing' stamp: 'jf 1/14/2009 13:35'!
handleFiltered: aRequestContext
	(self shouldTerminate: aRequestContext)
		ifTrue: [ self terminate: aRequestContext ]
		ifFalse: [ self critical: [ super handleFiltered: aRequestContext ] ]! !

!WAMutualExclusionFilter methodsFor: 'initialization' stamp: 'jf 1/25/2009 12:03'!
initialize
	super initialize.
	semaphore := WAPlatform current semaphoreClass forMutualExclusion! !

!WAMutualExclusionFilter methodsFor: 'private' stamp: 'jf 1/14/2009 13:34'!
shouldTerminate: aRequestContext
	^ (aRequestContext request fields includesKey: 'terminate')
		and: [ aRequestContext application preferenceAt: #sessionAllowTermination ]! !

!WAMutualExclusionFilter methodsFor: 'private' stamp: 'jf 1/22/2009 17:20'!
terminate: aRequestContext
	self terminateOwner.
	aRequestContext response
		contentType: WAMimeType textPlain;
		nextPutAll: 'Process terminated'.
	aRequestContext respond! !

!WAMutualExclusionFilter methodsFor: 'private' stamp: 'jf 1/22/2009 17:21'!
terminateOwner
	owner isNil ifFalse: [ WAPlatform current terminateProcess: owner ]! !

!WARequestFilter methodsFor: '*seaside-session' stamp: 'lr 8/31/2008 14:45'!
updateStates: aSnapshot
	aSnapshot register: next! !

!WAPackage class methodsFor: '*seaside-session' stamp: 'lr 10/23/2008 21:44'!
seasideSession
	^ self new
		name: 'Seaside-Session';
		description: 'Support for stateful and secure session handling.';
		addDependency: 'Seaside-Core';
		addDependency: 'Seaside-Squeak-Continuation';
		yourself! !

WASystemConfiguration subclass: #WAApplicationConfiguration
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Session'!

!WAApplicationConfiguration commentStamp: 'jf 7/22/2008 12:40' prior: 0!
WAApplicationConfiguration defines attributes required by WAApplication. All applications should include this configuration (this is done by default for new applications).

See #addAttributes: for details on provided attributes.!

!WAApplicationConfiguration methodsFor: 'description' stamp: 'lr 1/21/2009 18:04'!
describeOn: config
	(config string: #charSet)
		beAdvanced;
		label: 'Charater Set';
		comment: 'The character set reported to the browser. You may additionally need to configure your server that this is also the character set used by your server.';
		default: 'utf-8'.
	(config collection: #libraries)
		options: [ self libraryClasses ].
	(config string: #mimeType)
		beAdvanced;
		label: 'MIME Type';
		comment: 'The media type of HTML documents reported to the browser.';
		default: 'text/html'.
	(config string: #resourceBaseUrl)
		group: #server;
		comment: 'Sets the base URL for URLS generated with #resourceUrl:'.
	(config list: #sessionClass)
		comment: 'Sessions in this application will be instances of this class.';
		options: [ self sessionClasses ];
		default: WASession.
! !

!WAApplicationConfiguration methodsFor: 'description' stamp: 'lr 1/10/2009 15:20'!
libraryClasses
	^ WAFileLibrary allSubclasses asSortedCollection: [ :a :b | a name <= b name ]! !

!WAApplicationConfiguration methodsFor: 'ancestry' stamp: 'jf 7/29/2008 12:28'!
parents
	^ Array with: WARequestHandlingConfiguration instance! !

!WAApplicationConfiguration methodsFor: 'description' stamp: 'jf 7/22/2008 12:00'!
sessionClasses
	^ WASession withAllSubclasses asSortedCollection: [ :a :b | a name <= b name ]! !

WASystemConfiguration subclass: #WASessionConfiguration
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Session'!

!WASessionConfiguration commentStamp: 'jf 7/22/2008 12:30' prior: 0!
WASessionConfiguration defines attributes required by WASession and its subclasses. Any application using a WASession should include this configuration (this is done by default for new applications).

See #addAttributes: for details on provided attributes.!

!WASessionConfiguration methodsFor: 'description' stamp: 'lr 1/2/2009 20:56'!
describeOn: config
	(config list: #exceptionHandler)
		beAdvanced;
		options: [ self exceptionHandlerClasses ];
		default: WAErrorHandler.
	(config list: #mainClass)
		beAdvanced;
		options: [ self mainClasses ].
	(config boolean: #useSessionCookie)
		comment: 'Whether the session identifier should be stored in a cookie instead of the URL. Will fall back to URL if the client does not support cookies.';
		default: false.
	(config boolean: #sessionAllowTermination)
		comment: 'Whether sessions allows a special HTTP request to terminate the process currently running in the session. This is used during development and should be disabled for deployment.';
		default: false! !

!WASessionConfiguration methodsFor: 'description' stamp: 'jf 9/17/2008 20:10'!
exceptionHandlerClasses
	^ WAExceptionHandler withAllSubclasses asSortedCollection: [ :a :b | a name <= b name ]! !

!WASessionConfiguration methodsFor: 'description' stamp: 'lr 11/18/2007 15:18'!
mainClasses
	^ WAMain allSubclasses asSortedCollection: [ :a :b | a name <= b name ]! !

!WASessionConfiguration methodsFor: 'ancestry' stamp: 'jf 8/5/2008 08:49'!
parents
	^ Array with: WARequestHandlingConfiguration instance! !

WARequestHandler subclass: #WASession
	instanceVariableNames: 'continuations jumpTo cookiesEnabled'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Session'!

!WASession commentStamp: 'pmm 1/24/2009 10:24' prior: 0!
I am a Seaside session. A new instance of me gets created when an user accesses an application for the first time and is persistent as long as the user is interacting with it.

This class is intended to be subclasses by applications that need global state, like a user. Custom state can be added by creating instance variables and storing it there. The session can be retrieved by #session if inside a component or task or by evaluating: WACurrentRequestContext session

If the session has not been used for #defaultTimeoutSeconds, it is garbage collected by the system. To manually expire a session call #expire.

A good way to clear all sessions is the following code:

WARegistry clearAllHandlers.
WAPlatform current garbageCollect

Instance Variables
	cookiesEnabled:		<Boolean>
			
cookiesEnabled
	- Whether the client has a session cookie. This can only be true if #useSessionCookie is enabled.!

!WASession methodsFor: 'accessing' stamp: 'lr 4/15/2007 15:55'!
actionField
	^ '_k'! !

!WASession methodsFor: 'accessing' stamp: 'pmm 1/23/2009 21:02'!
actionUrlForContinuation: aContinuation
	^ self actionUrlForKey: (continuations store: aContinuation)! !

!WASession methodsFor: 'accessing' stamp: 'jf 10/27/2008 13:39'!
actionUrlForKey: aString
	^ self url
		addParameter: self actionField value: aString;
		yourself! !

!WASession methodsFor: 'deprecated' stamp: 'jf 1/22/2009 22:41'!
addLoadScript: aString
	self
		seasideDeprecatedApi: 'WASession>>#addLoadScript:'
		details: 'Use WAHtmlDocument>>#addLoadScript:. For example: "html document addLoadScript:".'.
		
	"We don't have access to the Canvas here so we can't provide an implementation"! !

!WASession methodsFor: 'deprecated' stamp: 'lr 12/30/2008 12:34'!
baseUrl
	self
		seasideDeprecatedApi: 'WASession>>#baseUrl'
		details: 'Use WAApplication>>#baseUrl.'.
	^ self application baseUrl! !

!WASession methodsFor: 'scripts' stamp: 'lr 4/15/2007 15:55'!
clearJumpTo
	jumpTo := nil! !

!WASession methodsFor: 'deprecated' stamp: 'lr 12/30/2008 12:35'!
closePopup
	self
		seasideDeprecatedApi: 'WASession>>#closePopup'
		details: 'Use WARequestContext>>#closeThisPopupAndReloadOpener.'.
	^ self requestContext closeThisPopupAndReloadOpener! !

!WASession methodsFor: 'deprecated' stamp: 'lr 12/30/2008 12:34'!
closePopupWithoutReloadingOpener
	self
		seasideDeprecatedApi: 'WASession>>#closePopupWithoutReloadingOpener'
		details: 'Use WARequestContext>>#closeThisPopup.'.
	^ self requestContext closeThisPopup! !

!WASession methodsFor: 'initializing' stamp: 'jf 1/10/2009 15:36'!
createCache
	"To configure the continuation cache you must currently subclass WASession and
	override this method."
	^ WACache new
		setExpiryPolicy: (WALeastRecentlyUsedExpiryPolicy new);
		yourself.! !

!WASession methodsFor: 'deprecated' stamp: 'lr 12/30/2008 12:35'!
currentRequest
	"Returns the current request object."

	self
		seasideDeprecatedApi: 'WASession>>#currentRequest'
		details: 'Use WARequestContext>>#request directly. For example: "self requestContext request".'.
	^ self requestContext request! !

!WASession methodsFor: 'deprecated' stamp: 'lr 12/30/2008 12:34'!
expire
	self
		seasideDeprecatedApi: 'WASession>>#expire'
		details: 'This method might be reimplemented again. In the meantim, if you just want to remove the Session from the Application, use WASession>>unregister (#unregistered will be called as a notification instead of #expired). Otherwise you should consider adding a Request Filter to the Session that implements whatever behaviour you want in order to block access to the Session.'.
	^ self unregister! !

!WASession methodsFor: 'processing' stamp: 'pmm 1/23/2009 19:39'!
handleFiltered: aRequestContext
	| key continuation |
	self handleSessionCookie: aRequestContext.
	WACurrentRequestContext use: aRequestContext during: [
		key := aRequestContext request fields
			at: self actionField
			ifAbsent: [ ^ self start ].
		(key isString not and: [ key isSequenceable ])
			ifTrue: [ key := key first ].
			
		continuation := continuations
			at: key
			ifAbsent: [ ^ self unknownRequest ].
		continuation value ]! !

!WASession methodsFor: 'private-cookies' stamp: 'pmm 1/24/2009 10:27'!
handleSessionCookie: aRequestContext
	"This method encapsulates all the session cookie detection logic.
	1. If the application doesn't want session cookies or we already have session cookies we do nothing.
	2. If we have a session cookie we note that in the cookiesEnabled instance variable.
	3. Otherwise we add a session cookie. In the next request this method will be executed. So the cookie is always added when the application wants a session coookie but the client doesn't support cookies.
	
	Wow, this method has almost more comment than code, clearly a smell."
	| request cookie |
	"1."
	(self useSessionCookie not or: [ cookiesEnabled ])
		ifTrue: [ ^self ].
		
	"2."
	request := aRequestContext request.
	cookie := self application sessionCookieFor: request.
	cookie isNil ifFalse: [
		cookiesEnabled := true.
		^self ].

	"3."
	aRequestContext response addCookie: self sessionCookie! !

!WASession methodsFor: 'initializing' stamp: 'jf 1/10/2009 15:35'!
initialize
	super initialize.
	self initializeFilters.
	continuations := self createCache.
	cookiesEnabled := false! !

!WASession methodsFor: 'initializing' stamp: 'jf 1/25/2009 12:28'!
initializeFilters
	self addFilter: WAMutualExclusionFilter new.
	self addFilter: WAExceptionFilter new! !

!WASession methodsFor: 'testing' stamp: 'jf 7/25/2008 20:33'!
isSession
	^ true! !

!WASession methodsFor: 'scripts' stamp: 'lr 4/15/2007 15:55'!
jumpToAnchor: aString
	jumpTo := aString! !

!WASession methodsFor: 'accessing' stamp: 'jf 10/27/2008 14:25'!
key
	^ self application keyFor: self! !

!WASession methodsFor: 'configuration' stamp: 'jf 7/28/2008 17:07'!
mainClass
	^ self preferenceAt: #mainClass! !

!WASession methodsFor: 'deprecated' stamp: 'lr 12/30/2008 12:35'!
newSessionUrl
	self
		seasideDeprecatedApi: 'WASession>>#newSessionUrl'
		details: 'Use WAApplication>>baseUrl.'.
	^ self application baseUrl! !

!WASession methodsFor: 'copying' stamp: 'jf 11/3/2008 08:57'!
postCopy
	super postCopy.
	continuations := continuations copy! !

!WASession methodsFor: 'configuration' stamp: 'jf 7/28/2008 17:06'!
preferenceAt: aSymbol
	^ self application isNil ifFalse: [ self application preferenceAt: aSymbol ]! !

!WASession methodsFor: 'deprecated' stamp: 'lr 12/30/2008 12:34'!
redirect
	self 
		seasideDeprecatedApi: 'WASession>>#redirect'
		details: 'Use WARequestContext>>#redirect.'.
	^ self requestContext redirect! !

!WASession methodsFor: 'deprecated' stamp: 'jf 1/21/2009 10:56'!
redirectResponseFor: aUrlOrString
	self
		seasideDeprecatedApi: 'WASession>>#redirectResponseFor:'
		details: 'Use WARequestContext>>redirectTo:.'.
	^ self requestContext redirectTo: aUrlOrString! !

!WASession methodsFor: 'deprecated' stamp: 'lr 12/30/2008 12:36'!
redirectTo: aUrlOrString
	self
		seasideDeprecatedApi: 'WASession>>#redirectTo:'
		details: 'Use WARequestContext>>#redirectTo:.'.
	^ self requestContext redirectTo: aUrlOrString! !

!WASession methodsFor: 'deprecated' stamp: 'lr 1/25/2009 12:00'!
redirectWithCookie: aCookie
	self 
		seasideDeprecatedApi: 'WASession>>#redirectWithCookie:'
		details: 'Do self requestContent response #addCooke:.'.
	^ self requestContext redirectWithCookie: aCookie! !

!WASession methodsFor: 'deprecated' stamp: 'lr 12/30/2008 12:53'!
redirectWithMessage: aString delay: aNumber 
	self
		seasideDeprecatedApi: 'WASession>>#redirectWithMessage:delay:'
		details: 'Use WARequestContext>>#redirectWithMessage:delay:.'.
	^ self requestContext redirectWithMessage: aString delay: aNumber! !

!WASession methodsFor: 'deprecated' stamp: 'lr 12/30/2008 12:35'!
respond: aBlock 
	self
		seasideDeprecatedApi: 'WASession>>#respond:'
	 	details: 'Use WARequestContext>>#respondAndContinue:.'.
	^ self requestContext respondAndContinue: aBlock! !

!WASession methodsFor: 'configuration' stamp: 'jf 8/5/2008 08:46'!
sessionAllowTermination
	^ self preferenceAt: #sessionAllowTermination! !

!WASession methodsFor: 'private' stamp: 'pmm 1/23/2009 21:03'!
sessionCookie
	"creates a new session cookie"
	^ self requestContext newCookie
		key: self application handlerCookieName;
		value: self key;
		discard: true;
		comment: 'the Seaside Session Cookie';
		commentUrl: 'http://www.seaside.st/documentation/sessionCookie';
		yourself! !

!WASession methodsFor: 'processing' stamp: 'pmm 1/23/2009 19:31'!
start
	self mainClass new start! !

!WASession methodsFor: 'processing' stamp: 'jf 10/19/2008 12:30'!
unknownRequest
	^ self start! !

!WASession methodsFor: 'expiring' stamp: 'jf 10/27/2008 11:29'!
unregister
	self application unregister: self! !

!WASession methodsFor: 'updating' stamp: 'jf 9/22/2008 13:02'!
updateRoot: anHtmlRoot
	self application updateRoot: anHtmlRoot.
	anHtmlRoot meta contentType: self application contentType.
	anHtmlRoot meta contentScriptType: WAMimeType textJavascript.
	anHtmlRoot htmlAttributes
		at: 'xmlns' put: 'http://www.w3.org/1999/xhtml';
		at: 'xml:lang' put: 'en';
		at: 'lang' put: 'en'.
	anHtmlRoot beXhtml10Strict.
	anHtmlRoot title: 'Seaside'! !

!WASession methodsFor: 'updating' stamp: 'lr 8/31/2008 14:49'!
updateStates: aSnapshot
	| current |
	current := self filter.
	[ current = self ] whileFalse: [
		current updateStates: aSnapshot.
		current := current next ].
	aSnapshot register: filter! !

!WASession methodsFor: 'updating' stamp: 'lr 4/15/2007 16:27'!
updateUrl: anUrl
	self application updateUrl: anUrl! !

!WASession methodsFor: 'accessing' stamp: 'jf 10/27/2008 21:24'!
url
	| url |
	url := cookiesEnabled
			ifTrue: [ self application baseUrl ]
			ifFalse: [ self application urlFor: self ].
	url takeServerParametersFromRequest: self requestContext request.
	jumpTo ifNotNil: [ url fragment: jumpTo ].
	^ url! !

!WASession methodsFor: 'configuration' stamp: 'jf 10/27/2008 21:23'!
useSessionCookie
	^ self preferenceAt: #useSessionCookie! !

WARegistry subclass: #WAApplication
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Session'!

!WAApplication commentStamp: 'rew 10/19/2007 10:47' prior: 0!
WAApplication is the starting point for a Seaside application. When a WAComponent is registered as a top level component a WAApplication object is added to a WADispatcher. The dispatcher forwards all requests to the WAApplication, which in turn forwards them to the correct WASession object. WAApplication's parent class WARegistry maintains a list of all active sessions to the application. 

"configuration" contains a chain of WAConfituration classes that define attributes of the application. The attribute "rootComponent", for example, defines the top level WAComponent class for the application. The configuration chain includes WAUserConfiguration, WAGlobalConfiguration, WARenderLoopConfiguration and WASessionConfiguration. Other configurations can be added to the chain when the top level application is registered with a dispatcher. (See below)

"libraries" is a collection of WALibrary classes, which are used to serve css, javascript and images used by the application. These may be in methods or in files. Sometimes these libraries are replaced by static files served by Apache. See WAFileLibrary class comment for more information.

Registering an Application.
	An application can be registered with a dispatcher by using the Seaside configuration page or via code. Below MyComponent is a subclass of WAComponent. The following registers the component as an application, gives some values to attributes (or preferences) and adds a library and a configuration. 

MyComponent class>>initialize
	"self initialize"
	| application |
	application := self registerAsApplication: 'sample'.
	application preferenceAt: #sessionClass put: Glorp.WAGlorpSession.
	application addLibrary: SampleLibrary.
	application configuration addAncestor: GlorpConfiguration new.
	application preferenceAt: #glorpDatabasePlatform put: Glorp.PostgreSQLPlatform.
	application preferenceAt: #databaseServer put: '127.0.0.1'.
	application preferenceAt: #databaseConnectString put: 'glorptests'.

MyComponent>>someInstanceMethod
	"example of how to access attributes (preferences)"
	self session application preferenceAt: #glorpDatabasePlatform!

!WAApplication class methodsFor: 'accessing' stamp: 'lr 11/16/2006 18:38'!
description
	^ 'Application'! !

!WAApplication methodsFor: 'libraries' stamp: 'lr 11/18/2007 15:24'!
addLibrary: aLibraryClass
	self preferenceAt: #libraries put: (self libraries copyWith: aLibraryClass)! !

!WAApplication methodsFor: 'configuration' stamp: 'pmm 4/6/2008 15:11'!
charSet
	^ self preferenceAt: #charSet! !

!WAApplication methodsFor: 'configuration' stamp: 'jf 9/22/2008 12:54'!
contentType
	^ self mimeType seasideMimeType
		charset: self charSet;
		yourself! !

!WAApplication methodsFor: 'configuration' stamp: 'MAD 9/13/2008 15:02'!
defaultConfiguration
	^ super defaultConfiguration
		removeParent: WARequestHandlingConfiguration instance;
		addParent: WAApplicationConfiguration instance;
		addParent: WASessionConfiguration instance;
		yourself! !

!WAApplication methodsFor: 'request handling' stamp: 'lr 1/12/2009 08:36'!
handleDefaultRequest: aRequestContext
	| session |
	session := self sessionClass new.
	self register: session.
	session handle: aRequestContext! !

!WAApplication methodsFor: 'private' stamp: 'lr 1/12/2009 09:14'!
handleExpiredRequest: aRequestContext
	| sessionCookie |
	sessionCookie := self sessionCookieFor: aRequestContext request.
	sessionCookie isNil 
		ifFalse: [ aRequestContext response deleteCookie: sessionCookie path: self baseUrl pathString ].
	super handleExpiredRequest: aRequestContext! !

!WAApplication methodsFor: 'processing' stamp: 'lr 1/19/2009 18:23'!
handleFiltered: aRequestContext
	"WAApplication instances only understand GET and POST requests."

	(self isImplemented: aRequestContext request method)
		ifTrue: [ super handleFiltered: aRequestContext ]
		ifFalse: [ aRequestContext responseFactory notImplemented; respond ]! !

!WAApplication methodsFor: 'accessing' stamp: 'pmm 3/22/2008 15:13'!
handlerCookieName
	"returns the key of the session cookie"
	^ self handlerField! !

!WAApplication methodsFor: 'processing' stamp: 'jf 10/7/2008 18:14'!
handlerField
	^ '_s'! !

!WAApplication methodsFor: 'testing' stamp: 'mb 4/5/2006 20:26'!
isApplication
	^true! !

!WAApplication methodsFor: 'private' stamp: 'lr 1/12/2009 08:40'!
isImplemented: aString
	"Answer whether the receiver implements the given HTTP method."

	^ #('GET' 'POST') includes: aString! !

!WAApplication methodsFor: 'request handling' stamp: 'lr 1/7/2009 12:25'!
keyFromContext: aRequestContext ifAbsent: aBlock
	| cookie |
	^ super
		keyFromContext: aRequestContext
		ifAbsent: [
			cookie := self sessionCookieFor: aRequestContext request.
			cookie isNil
				ifFalse: [ cookie value ]
				ifTrue: [ aBlock value ] ]! !

!WAApplication methodsFor: 'configuration' stamp: 'lr 2/10/2008 15:59'!
libraries
	^ self preferenceAt: #libraries! !

!WAApplication methodsFor: 'configuration' stamp: 'pmm 4/6/2008 15:11'!
mimeType
	^ self preferenceAt: #mimeType! !

!WAApplication methodsFor: 'libraries' stamp: 'lr 11/18/2007 15:25'!
removeLibrary: aLibraryClass
	self preferenceAt: #libraries put: (self libraries copyWithout: aLibraryClass)! !

!WAApplication methodsFor: 'configuration' stamp: 'lr 2/12/2007 22:02'!
resourceBaseUrl
	^ self preferenceAt: #resourceBaseUrl! !

!WAApplication methodsFor: 'configuration' stamp: 'lr 10/28/2007 14:42'!
sessionClass
	^ self preferenceAt: #sessionClass! !

!WAApplication methodsFor: 'private' stamp: 'jf 9/22/2008 21:08'!
sessionCookieFor: aRequest
	"answers the session cookie or nil
	There can be several cookies with the correct key (_s). We try to find the one who has the most specific path."

	| candidates requestUrl |
	candidates := aRequest cookiesAt: self handlerCookieName.
	
	candidates isEmptyOrNil
		ifTrue: [ ^nil ].
	candidates size = 1 
		ifTrue: [ ^candidates first ].
		
	requestUrl := aRequest url.
	candidates := candidates collect: [ :each |
		each -> (each path seasideUrl relativeTo: requestUrl) ].
	
	candidates := candidates reject: [ :each |
		| path |
		path := each value.
		path notEmpty and: [ path first = '..' ] ].

	candidates isEmpty
		ifTrue: [  ^nil ].
		
	candidates size > 1
		ifTrue: [
			candidates sort: [ :a :b |
				 a value size <= b value size ] ].
	
	^candidates first key! !

!WAApplication methodsFor: 'request handling' stamp: 'lr 4/9/2007 10:51'!
updateRoot: anHtmlRoot
	self libraries do: [ :each | each default updateRoot: anHtmlRoot ]! !

!WAApplication methodsFor: 'request handling' stamp: 'lr 4/15/2007 16:21'!
updateUrl: anUrl! !
